﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

namespace Chaikin___SampleApp
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        // Global variables:
        int noIter; // number of iterations
        List<Point> ControlVertices; // list of control vertices
        public MainWindow()
        {
            InitializeComponent();
            noIter = Convert.ToInt32(NumberIter.Text);  // conversion from a string to integer
            ControlVertices = new List<Point>(); // List is a reference type, so we need to create it 
        }

        public void DrawPoints(List<Point> CV, Canvas C)
        {
            for (int i = 0; i < CV.Count; i++)
            {
                // point will be depicted as red circle
                Ellipse E = new Ellipse();

                // color of the circle
                E.Fill = new SolidColorBrush(Colors.IndianRed);

                // proportions
                E.Width = 4;
                E.Height = 4;

                // position of the left top corner of the circle on the Canvas
                Canvas.SetLeft(E, CV[i].X - 2);
                Canvas.SetTop(E, CV[i].Y - 2);

                // z-index determines the order of the elements in the canvas, top objects have the highest z-index
                Canvas.SetZIndex(E, 5);

                // now add the circle to the canvas
                C.Children.Add(E);
            }
        }

        public void DrawLines(List<Point> CV, Canvas C)
        {
            for (int i = 0; i < CV.Count - 1; i++)
            {
                Line L = new Line();

                // color and width of the line
                L.Stroke = new SolidColorBrush(Colors.YellowGreen);
                L.StrokeThickness = 1;

                // endpoints of the line
                L.X1 = CV[i].X;
                L.Y1 = CV[i].Y;

                L.X2 = CV[i + 1].X;
                L.Y2 = CV[i + 1].Y;


                // z-index determines the order of the elements in the canvas, top objects have the highest z-index
                Canvas.SetZIndex(L, 4);

                // now add the line to the canvas
                C.Children.Add(L);
            }
        }

        public void DrawChaikin(List<Point> CV, int noi, Canvas C)
        {
            if (noi > 0) // for desired numbers of iteration
            {
                List<Point> Refined = new List<Point>();
                for (int i = 0; i < CV.Count - 1; i++)
                {
                    // create a new list of vertices of the "iterated" curve by given equations
                    Point
                        P = new Point(),
                        R = new Point(); 
                    P.X = 3 * CV[i].X / 4 + CV[i + 1].X / 4;
                    P.Y = 3 * CV[i].Y / 4 + CV[i + 1].Y / 4;

                    R.X = CV[i].X / 4 + 3 * CV[i + 1].X / 4;
                    R.Y = CV[i].Y / 4 + 3 * CV[i + 1].Y / 4;

                    Refined.Add(P);
                    Refined.Add(R);
                }
                // now proceed to next iteration (noi--) calling the same function to the different set of vertices "Refined"
                DrawChaikin(Refined, noi - 1, C);
            }
            else if (noi == 0) // when noi = 0, the curve was refined the desired number of times, that means we can draw it
            {
                // we draw the curve as a sequence of lines connecting consecutive vertices
                for (int i = 0; i < CV.Count - 1; i++)
                {
                    Line L = new Line();

                    // color and width of the line
                    L.Stroke = new SolidColorBrush(Colors.MediumTurquoise);
                    L.StrokeThickness = 2;

                    // endpoints of the line
                    L.X1 = CV[i].X;
                    L.Y1 = CV[i].Y;

                    L.X2 = CV[i + 1].X;
                    L.Y2 = CV[i + 1].Y;

                    // z-index determines the order of the elements in the canvas, top objects have the highest z-index
                    Canvas.SetZIndex(L, 3);

                    // now add the line to the canvas
                    C.Children.Add(L);
                }
            }
            else return;
        }

        private void IncreaseIter_Click(object sender, RoutedEventArgs e)
        {
            // by clicking on the "+" button we want to increase the number of iterations, redraw the curve and change the number in textbox 
            noIter++;
            NumberIter.Text = Convert.ToString(noIter); // we need to convert the integer value to string

            // before redrawing a curve, we need to clear the canvas
            g.Children.Clear();

            // draw a point depicting the inserted vertex on the canvas "g"
            if (ControlVertices.Count >= 1) DrawPoints(ControlVertices, g);

            // if there are at least two vertices, draw the line conecting the consecutive veritces on the canvas "g"
            if (ControlVertices.Count >= 2) DrawLines(ControlVertices, g);

            //if there are at least three points, we can start drawing the Chaikin's curve with given number of iterations
            if (ControlVertices.Count >= 3) DrawChaikin(ControlVertices, noIter, g);
        }

        private void DecreaseIter_Click(object sender, RoutedEventArgs e)
        {
            // by clicking on the "-" button we want to decrease the number of iterations, but this must not be smaller than 0
            if (noIter > 0)
            {
                noIter--;
                NumberIter.Text = Convert.ToString(noIter); // we need to convert the integer value to string

                // before redrawing a curve, we need to clear the canvas
                g.Children.Clear();

                // draw a point depicting the inserted vertex on the canvas "g"
                if (ControlVertices.Count >= 1) DrawPoints(ControlVertices, g);

                // if there are at least two vertices, draw the line conecting the consecutive veritces on the canvas "g"
                if (ControlVertices.Count >= 2) DrawLines(ControlVertices, g);

                //if there are at least three points, we can start drawing the Chaikin's curve with given number of iterations
                if (ControlVertices.Count >= 3) DrawChaikin(ControlVertices, noIter, g);
            }
        }

        private void g_MouseDown(object sender, MouseButtonEventArgs e)
        {
            // add a point under the cursor to the list of control vertices
            ControlVertices.Add(e.GetPosition(g)); // GetPoistion returns a Point type, we need to declare what element is the position relative to. In our case it is the canvas named "g"

            // before redrawing a curve, we need to clear the canvas
            g.Children.Clear();

            // draw a point depicting the inserted vertex on the canvas "g"
            if (ControlVertices.Count >= 1) DrawPoints(ControlVertices, g);

            // if there are at least two vertices, draw the line conecting the consecutive veritces on the canvas "g"
            if (ControlVertices.Count >= 2) DrawLines(ControlVertices, g);

            //if there are at least three points, we can start drawing the Chaikin's curve with given number of iterations
            if (ControlVertices.Count >= 3) DrawChaikin(ControlVertices, noIter, g);
        }

        private void Reset_Click(object sender, RoutedEventArgs e)
        {
            g.Children.Clear();
            ControlVertices.Clear();
        }
    }
}
